#include "image_roi_alg_node.h"

ImageRoiAlgNode::ImageRoiAlgNode(void) :
  algorithm_base::IriBaseAlgorithm<ImageRoiAlgorithm>(),
  image_out_camera_manager(ros::NodeHandle("~image_out")),
  it(this->private_node_handle_)
{
  //init class attributes if necessary
  if(!this->private_node_handle_.getParam("rate", this->config_.rate))
  {
    ROS_WARN("ImageRoiAlgNode::ImageRoiAlgNode: param 'rate' not found");
  }
  else
    this->setRate(this->config_.rate);

  // [init publishers]
  this->image_out_publisher_ = this->it.advertiseCamera("image_out/image_raw", 1);
  
  // [init subscribers]
  this->image_in_subscriber_ = this->it.subscribeCamera("image_in/image_raw", 1, &ImageRoiAlgNode::image_in_callback, this);
  pthread_mutex_init(&this->image_in_mutex_,NULL);

  
  // [init services]
  
  // [init clients]
  
  // [init action servers]
  
  // [init action clients]
}

ImageRoiAlgNode::~ImageRoiAlgNode(void)
{
  // [free dynamic memory]
  pthread_mutex_destroy(&this->image_in_mutex_);
}

void ImageRoiAlgNode::mainNodeThread(void)
{
  //lock access to algorithm if necessary
  this->alg_.lock();
  ROS_DEBUG("ImageRoiAlgNode::mainNodeThread");
  // [fill msg structures]
  
  // [fill srv structure and make request to the server]
  
  // [fill action structure and make request to the action server]

  // [publish messages]

  
  this->alg_.unlock();
}

/*  [subscriber callbacks] */
void ImageRoiAlgNode::image_in_callback(const sensor_msgs::Image::ConstPtr& msg, const sensor_msgs::CameraInfoConstPtr& info)
{
  cv::Rect roi_rect;
  sensor_msgs::CameraInfo new_info;

  ROS_DEBUG("ImageRoiAlgNode::image_in_callback: New Message Received");

  //use appropiate mutex to shared variables if necessary
  this->alg_.lock();
  //this->image_in_mutex_enter();

  //std::cout << msg->data << std::endl;
  // Uncomment the following line to convert the input image to OpenCV format
  this->cv_image_ = cv_bridge::toCvCopy(msg, this->config_.output_format);
  roi_rect.x = this->config_.roi_x_offset;
  if(roi_rect.x<0)
    roi_rect.x=0;
  else if(roi_rect.x>=info->width)
    roi_rect.x=info->width-1;
  roi_rect.y = this->config_.roi_y_offset;
  if(roi_rect.y<0)
    roi_rect.y=0;
  else if(roi_rect.y>=info->height)
    roi_rect.y=info->height-1;
  roi_rect.width = this->config_.roi_width;
  if(roi_rect.x+roi_rect.width>info->width)
    roi_rect.width=info->width-roi_rect.x;
  roi_rect.height = this->config_.roi_height;
  if(roi_rect.y+roi_rect.height>info->height)
    roi_rect.height=info->height-roi_rect.y;
  this->cv_image_->image=this->cv_image_->image(roi_rect);
  this->image_out_Image_msg_=*this->cv_image_->toImageMsg();
  new_info=*info;
  new_info.height=roi_rect.height;
  new_info.width=roi_rect.width;
  new_info.P[6]=roi_rect.height/2.0;
  new_info.K[5]=roi_rect.height/2.0;
  this->image_out_Image_msg_.height=roi_rect.height;
  this->image_out_Image_msg_.width=roi_rect.width;
  this->image_out_publisher_.publish(this->image_out_Image_msg_,new_info);

  //unlock previously blocked shared variables
  this->alg_.unlock();
  //this->image_in_mutex_exit();
}

void ImageRoiAlgNode::image_in_mutex_enter(void)
{
  pthread_mutex_lock(&this->image_in_mutex_);
}

void ImageRoiAlgNode::image_in_mutex_exit(void)
{
  pthread_mutex_unlock(&this->image_in_mutex_);
}


/*  [service callbacks] */

/*  [action callbacks] */

/*  [action requests] */

void ImageRoiAlgNode::node_config_update(Config &config, uint32_t level)
{
  this->alg_.lock();
  if(config.rate!=this->getRate())
    this->setRate(config.rate);
  this->config_=config;
  this->alg_.unlock();
}

void ImageRoiAlgNode::addNodeDiagnostics(void)
{
}

/* main function */
int main(int argc,char *argv[])
{
  return algorithm_base::main<ImageRoiAlgNode>(argc, argv, "image_roi_alg_node");
}
